# Generated by Django 2.0.9 on 2020-06-20 06:08

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DeliveryMethod',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('code', models.CharField(max_length=255, unique=True, verbose_name='Code')),
            ],
            options={
                'verbose_name': 'Delivery method',
                'verbose_name_plural': 'Delivery methods',
            },
        )
    ]




import sys

from modeltranslation.translator import translator

from delivery.models import Warehouse, Region, City, DeliveryMethod


translator.register(DeliveryMethod, fields=['name'])

if 'sync_translation_fields' not in sys.argv:
    translator.register(Region, fields=['name'])
    translator.register(City, fields=['name'])
    translator.register(Warehouse, fields=['name'])




from django.http import JsonResponse

from delivery.lib import search_warehouses, search_cities


def get_warehouses(request):

    warehouses = search_warehouses(
        request.GET.get('delivery_method'),
        request.GET.get('city'),
        request.GET.get('query'),
        limit=10)

    suggestions = [w.name for w in warehouses]

    return JsonResponse({
        'query': request.GET.get('query'),
        'suggestions': suggestions
    })


def get_cities(request):

    query = request.GET.get('query')

    suggestions = [str(c) for c in search_cities(query, limit=10)]

    return JsonResponse({
        'query': query,
        'suggestions': suggestions
    })




from django.contrib import admin

from modeltranslation.admin import TranslationAdmin

from delivery.models import DeliveryMethod


@admin.register(DeliveryMethod)
class DeliveryMethodAdmin(TranslationAdmin):

    list_display = ['name', 'code']




from django.forms.widgets import Widget


class DeliveryFormFieldWidget(Widget):

    template_name = 'delivery/form.html'

    def __init__(self, *args, **kwargs):
        self.form = None
        super().__init__(*args, **kwargs)

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        context['form'] = self.form
        return context




class DeliveryRouter(object):

    app_label = 'delivery'
    db_name = 'delivery'

    def db_for_read(self, model, **hints):

        if self._is_delivery_method_model(model):
            return None

        if model._meta.app_label == self.app_label:
            return self.db_name

        return None

    def db_for_write(self, model, **hints):

        if self._is_delivery_method_model(model):
            return None

        if model._meta.app_label == self.app_label:
            return self.db_name

        return None

    def allow_migrate(self, db, app_label, model_name=None, **hints):

        if model_name == 'deliverymethod':
            return None

        if app_label == self.app_label:
            return False

        return None

    def _is_delivery_method_model(self, model):
        return model.__name__.lower() == 'deliverymethod'




from django import forms
from django.utils.translation import ugettext_lazy as _

from delivery.models import DeliveryMethod


class DeliveryForm(forms.Form):

    delivery_method = forms.ModelChoiceField(
        label=_('Delivery method'),
        queryset=DeliveryMethod.objects.all())

    city = forms.CharField(
        label=_('City'),
        required=False
    )

    warehouse = forms.CharField(
        label=_('Warehouse'),
        required=False
    )

    @property
    def delivery_methods(self):
        return {m.code: m.id for m in DeliveryMethod.objects.all()}




from django.conf import settings
from django.forms import Field, ValidationError
from django.utils.translation import ugettext_lazy as _
from django.template.loader import render_to_string

from delivery.widgets import DeliveryFormFieldWidget
from delivery.forms import DeliveryForm


class DeliveryFormField(Field):

    widget = DeliveryFormFieldWidget
    form = None

    def init_form(self, *args, **kwargs):

        self.form = DeliveryForm(*args, **kwargs)
        self.widget.form = self.form

    def clean(self, *args, **kwargs):

        if not self.form.is_valid():
            raise ValidationError(_('Form is invalid.'))

        return self.form.cleaned_data

    def commit(self, *args, **kwargs):
        return self.form.commit(*args, **kwargs)

    def get_delivery_method(self):
        return self.form.cleaned_data['delivery_method']

    def get_address(self):
        return '{}, {}'.format(
            self.form.cleaned_data['warehouse'],
            self.form.cleaned_data['city'])

    @property
    def media(self):
        return render_to_string('delivery/media.html', {
            'STATIC_URL': settings.STATIC_URL,
            'form': self.form
        })





from django.urls import path, include

from django.conf.urls.i18n import i18n_patterns

from delivery import views


app_name = 'delivery'


urlpatterns = [

    path('cities', views.get_cities, name='cities'),

    path('warehouses', views.get_warehouses, name='warehouses')

]


app_urls = i18n_patterns(

    path('delivery/', include((urlpatterns, app_name))),

)




from django.db import models
from django.utils.translation import ugettext_lazy as _


class DeliveryMethod(models.Model):

    name = models.CharField(_('Name'), max_length=255)

    code = models.CharField(_('Code'), max_length=255, unique=True)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Delivery method')
        verbose_name_plural = _('Delivery methods')


class DeliveryMethodField(models.ForeignKey):

    def __init__(
            self,
            to=DeliveryMethod,
            verbose_name=_('Delivery method'),
            on_delete=models.CASCADE,
            null=True,
            *args, **kwargs):

        super().__init__(
            to,
            verbose_name=verbose_name,
            on_delete=on_delete,
            null=null,
            *args, **kwargs)


class Region(models.Model):

    name = models.CharField(_('Name'), max_length=255)

    reference = models.CharField(_('Reference'), max_length=255)

    def __str__(self):
        if self.reference == '71508128-9b87-11de-822f-000c2965ae0e':
            return self.name

        return '{} {}'.format(self.name, _('region'))

    class Meta:
        verbose_name = _('Region')
        verbose_name_plural = _('Regions')


class City(models.Model):

    region = models.ForeignKey(
        Region,
        verbose_name=_('Region'),
        related_name='cities',
        on_delete=models.CASCADE)

    name = models.CharField(_('Name'), max_length=255)

    reference = models.CharField(_('Reference'), max_length=255)

    def __str__(self):
        return '{} - {}'.format(self.name, self.region)

    class Meta:
        verbose_name = _('City')
        verbose_name_plural = _('Cities')


class Warehouse(models.Model):

    city = models.ForeignKey(
        City,
        verbose_name=_('City'),
        related_name='warehouses',
        on_delete=models.CASCADE)

    delivery_method = models.ForeignKey(
        DeliveryMethod,
        verbose_name=_('Delivery method'),
        on_delete=models.CASCADE)

    name = models.CharField(_('Name'), max_length=255, db_index=True)

    reference = models.CharField(_('Reference'), max_length=255)

    def __str__(self):
        return '{}, {}, {}'.format(self.delivery_method, self.city, self.name)

    class Meta:
        verbose_name = _('Warehouse')
        verbose_name_plural = _('Warehouses')




from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


def setup_settings(settings, is_prod, **kwargs):

    settings['DATABASES']['delivery'] = {
        'ENGINE': 'django.db.backends.postgresql',
        'NAME': 'mp',
        'USER': 'dev'
    }

    settings['DATABASE_ROUTERS'] = settings.get('DATABASE_ROUTERS', []) + [
        'delivery.routers.DeliveryRouter'
    ]


class DeliveryAppConfig(AppConfig):
    name = 'delivery'
    verbose_name = _('Delivery')


default_app_config = 'delivery.DeliveryAppConfig'




from model_search import model_search

from delivery.models import Warehouse, City, DeliveryMethod


def search_warehouses(delivery_method_id, city, query, limit=None):

    if not query or not city or not delivery_method_id:
        return []

    try:
        city = city.split(' - ')[0]
    except Exception:
        return []

    try:
        delivery_method = DeliveryMethod.objects.get(id=delivery_method_id)
    except DeliveryMethod.DoesNotExist:
        return []

    warehouses = Warehouse.objects.filter(
        delivery_method__code=delivery_method.code,
        city__name=city
    )

    queryset = model_search(query, warehouses, ['name'])

    if limit is not None:
        return queryset[:limit]

    return queryset


def search_cities(query, limit=None):

    if not query:
        return []

    queryset = model_search(query, City.objects.all(), ['name'])

    if limit is not None:
        return queryset[:limit]

    return queryset
